"""
Marblex API Wrapper
~~~~~~~~~~~~~~~~~~~
A basic wrapper for the Marblex API.
:copyright: (c) 2022-present xStacia
:license: MIT, see LICENSE for more details.
"""
__title__ = 'marblex.py'
__version__ = '1.3.2'
__author__ = 'xStacia'
__license__ = 'MIT'
__copyright__ = 'Copyright 2022-present xStacia'

from .client import *
from .models import *